import java.util.ArrayList;
public class FractionalNumber
{
  public FractionalNumber(int i)
  {
    this.numerator = new NaturalNumber(i);
    this.denominator = new NaturalNumber(1);
  }
  public FractionalNumber(int numerator, int denominator)
  {
    this.numerator = new NaturalNumber(numerator);
    this.denominator = new NaturalNumber(denominator);
  }
  public FractionalNumber(NaturalNumber numerator, NaturalNumber denominator)
  {
    this.numerator = numerator;
    this.denominator = denominator;
  }
  public FractionalNumber(FractionalNumber numerator, FractionalNumber denominator)
  {
    this.numerator = numerator.numerator.multiply(denominator.denominator);
    this.denominator = numerator.denominator.multiply(denominator.numerator);
  }
  public FractionalNumber invert()
  {
    return new FractionalNumber(denominator, numerator);
  }
  private NaturalNumber numerator;
  public NaturalNumber getNumerator()
  {
    return numerator;
  }
  private NaturalNumber denominator;
  public NaturalNumber getDenominator()
  {
    return denominator;
  }
  public FractionalNumber add(FractionalNumber other)
  {
    NaturalNumber newNumerator = numerator.multiply(other.denominator).add(other.numerator.multiply(denominator));
    NaturalNumber newDenominator = denominator.multiply(other.denominator);
    
    return new FractionalNumber(newNumerator, newDenominator);
  }
  public FractionalNumber multiply(FractionalNumber other)
  {
    NaturalNumber newNumerator = numerator.multiply(other.numerator);
    NaturalNumber newDenominator = denominator.multiply(other.denominator);
    
    return new FractionalNumber(newNumerator, newDenominator);
  }
  public FractionalNumber subtract(FractionalNumber other)
  {
    NaturalNumber newNumerator = numerator.multiply(other.denominator).subtract(other.numerator.multiply(denominator));
    NaturalNumber newDenominator = denominator.multiply(other.denominator);
    
    return new FractionalNumber(newNumerator, newDenominator);
  }
  public FractionalNumber pow(int other)
  {
    return new FractionalNumber(numerator.pow(other), denominator.pow(other));
  }
  public void print(int digits)
  {
    // Rather inefficient, but oh well.
    ArrayList<NaturalNumber> bases = new ArrayList<NaturalNumber>();
    bases.add(new NaturalNumber(1));
    while (!bases.get(bases.size()-1).multiply(denominator).isLargerThan(numerator))
    {
      bases.add(bases.get(bases.size()-1).multiply(10));
    }
    NaturalNumber approx = new NaturalNumber(0);
    for (int i = bases.size()-2; i >= 0; i--)
    {
      int count = 0;
      NaturalNumber newApprox = approx.add(bases.get(i));
      while (!newApprox.multiply(denominator).isLargerThan(numerator))
      {
        approx = newApprox;
        newApprox = newApprox.add(bases.get(i));
        count++;
      }
      System.out.print(count);
    }
    if (digits > 0)
    {
      System.out.print(".");
      NaturalNumber tempNumerator = numerator;
      NaturalNumber tempDenominator = denominator;
      for (int i = 0; i < digits; i++)
      {
        tempNumerator = tempNumerator.multiply(10);
        approx = approx.multiply(10);
        int count = 0;
        NaturalNumber newApprox = approx.add(1);
        while (!newApprox.multiply(tempDenominator).isLargerThan(tempNumerator))
        {
          approx = newApprox;
          newApprox = newApprox.add(1);
          count++;
        }
        System.out.print(count);
      }
    }
  }
  public void printSimplified()
  {
    NaturalNumber gcd = numerator.greatestCommonDivisor(denominator);
    new FractionalNumber(numerator, gcd).print(0);
    System.out.print("/");
    new FractionalNumber(denominator, gcd).print(0);
  }
}




















































