import java.util.ArrayList;
import java.lang.Math;
public class NaturalNumber
{
  public static final long RADIX = ((long)Integer.MAX_VALUE)+1;
  private ArrayList<Long> digits;
  private NaturalNumber()
  {
    digits = new ArrayList<Long>();
  }
  public NaturalNumber(int number)
  {
    digits = new ArrayList<Long>();
    append(number);
  }
  public NaturalNumber(ArrayList<Integer> digits)
  {
    this.digits = new ArrayList<Long>(digits.size());
    for (int i = 0; i < digits.size(); i++)
    {
      append(digits.get(i));
    }
  }
  public NaturalNumber add(int other)
  {
    return add(new NaturalNumber(other));
  }
  public NaturalNumber add(NaturalNumber other)
  {
    long carry = 0;
    int newSizeMin = Math.max(size(), other.size());
    NaturalNumber result = new NaturalNumber();
    for (int i = 0; i < newSizeMin; i++)
    {
      result.append(get(i) + other.get(i) + carry);
      if (result.get(i) >= RADIX)
      {
        result.set(i, result.get(i)-RADIX);
        carry = 1;
      }
      else
      {
        carry = 0;
      }
    }
    if (carry == 1)
    {
      result.append(1);
    }
    return result;
  }
  public NaturalNumber subtract(NaturalNumber other)
  {
    // THIS ONLY WORKS PROPERLY IF this >= other!!!
    long carry = 0;
    NaturalNumber result = new NaturalNumber();
    for (int i = 0; i < size(); i++)
    {
      result.append(get(i) - other.get(i) - carry);
      if (result.get(i) < 0)
      {
        result.set(i, result.get(i)+RADIX);
        carry = 1;
      }
      else
      {
        carry = 0;
      }
    }
    while (result.size() > 0 && result.get(result.size()-1) == 0)
    {
      result.digits.remove(result.size()-1);
    }
    return result;
  }
  public boolean isEven()
  {
    return get(0) % 2 == 0;
  }
  public NaturalNumber halve()
  {
    NaturalNumber result = new NaturalNumber();
    result.append(get(0)/2);
    for (int i = 1; i < size(); i++)
    {
      result.append(get(i)/2);
      if (get(i)%2 == 1)
      {
        result.set(i-1, result.get(i-1)+RADIX/2);
      }
    }
    if (result.get(result.size()-1) == 0)
    {
      result.digits.remove(result.size()-1);
    }
    return result;
  }
  public NaturalNumber multiply(int other)
  {
    return multiply(new NaturalNumber(other));
  }
  public NaturalNumber multiply(NaturalNumber other)
  {
    // This is actually rather slow. FFT or Karatsuba based multiplication is faster.
    if (isZero()) { return this; }
    if (other.isZero()) { return other; }
    int m = size();
    int n = other.size();
    NaturalNumber result = new NaturalNumber();
    result.set(m+n-1, 0); // fill with zeroes
    for (int i = 0; i < m; i++)
    {
      long carry = 0;
      for (int j = 0; j < n; j++)
      {
        long t = result.get(i+j) + get(i) * other.get(j) + carry;
        carry = t / RADIX;
        result.set(i+j, t - carry * RADIX);
      }
      result.set(i+n, carry);
    }
    if (result.digits.get(m+n-1) == 0)
    {
      result.digits.remove(m+n-1);
    }
    return result;
  }
  public NaturalNumber pow(int other)
  {
    // Exponentiation through repeated squaring.
    NaturalNumber result = new NaturalNumber(1);
    int n = 0;
    for (int i = 1; i <= other; i *= 2) n++;
    for (int i = n-1; i >= 0; i--)
    {
      result = result.multiply(result);
      if (other >= (1 << i))
      {
        other -= (1 << i);
        result = result.multiply(this);
      }
    }
    return result;
  }
  public NaturalNumber greatestCommonDivisor(NaturalNumber other)
  {
    NaturalNumber a = this;
    NaturalNumber b = other;
    NaturalNumber d = new NaturalNumber(1);
    while (a.isEven() && b.isEven())
    {
      a = a.halve();
      b = b.halve();
      d = d.multiply(2);
    }
    while (a.isEven())
    {
      a = a.halve();
    }
    while (b.isEven())
    {
      b = b.halve();
    }
    if (a.isSmallerThan(b))
    {
      NaturalNumber temp = a;
      a = b;
      b = temp;
    }
    while (!b.isZero())
    {
      a = a.subtract(b);
      while (!a.isZero() && a.isEven())
      {
        a = a.halve();
      }
      if (a.isSmallerThan(b))
      {
        NaturalNumber temp = a;
        a = b;
        b = temp;
      }
    }
    return d.multiply(a);
  }
  public long get(int i)
  {
    if (i >= size())
      return 0;
    return (long)digits.get(i);
  }
  private void append(long d)
  {
    digits.add(d);
  }
  private void set(int i, long d)
  {
    while (i > size())
    {
      append(0);
    }
    if (i >= size())
    {
      append(d);
    }
    else
    {
      digits.set(i, d);
    }
  }
  private int size()
  {
    return digits.size();
  }
  public boolean isZero()
  {
    return size() == 0 || (size() == 1 && get(0)==0);
  }
  public void print()
  {
    if (isZero())
    {
      System.out.print(0);
      return;
    }
    // Rather inefficient, could use division here, but division is annoying to code.
    ArrayList<NaturalNumber> bases = new ArrayList<NaturalNumber>();
    bases.add(new NaturalNumber(1));
    while (!bases.get(bases.size()-1).isLargerThan(this))
    {
      bases.add(bases.get(bases.size()-1).multiply(10));
    }
    NaturalNumber approx = new NaturalNumber(0);
    for (int i = bases.size()-2; i >= 0; i--)
    {
      int count = 0;
      NaturalNumber newApprox = approx.add(bases.get(i));
      while (!newApprox.isLargerThan(this))
      {
        approx = newApprox;
        newApprox = newApprox.add(bases.get(i));
        count++;
      }
      System.out.print(count);
    }
  }
  public boolean isLargerThan(NaturalNumber other)
  {
    for (int i = Math.max(size(), other.size())-1; i >= 0; i--)
    {
      if (get(i) != other.get(i))
        return get(i) > other.get(i);
    }
    return false;
  }
  public boolean isSmallerThan(NaturalNumber other)
  {
    for (int i = Math.max(size(), other.size())-1; i >= 0; i--)
    {
      if (get(i) != other.get(i))
        return get(i) < other.get(i);
    }
    return false;
  }
  public boolean isEqualTo(NaturalNumber other)
  {
    for (int i = Math.max(size(), other.size())-1; i >= 0; i--)
    {
      if (get(i) != other.get(i))
        return false;
    }
    return true;
  }
  private static ArrayList<NaturalNumber> cache = new ArrayList<NaturalNumber>();
  public static NaturalNumber factorial(int k)
  {
    if (k == 0) return new NaturalNumber(1);
    if (cache.size() == 0) { cache.add(new NaturalNumber(1)); }
    while (cache.size() < k)
    {
      cache.add(cache.get(cache.size()-1).multiply(cache.size()+1));
    }
    return cache.get(k-1);
  }
  public static NaturalNumber fallingFactorial(int x, int k)
  {
    NaturalNumber result = new NaturalNumber(x);
    NaturalNumber factor = new NaturalNumber(x);
    for (int i = 1; i < k; i++)
    {
      factor = factor.subtract(new NaturalNumber(1));
      result = result.multiply(factor);
    }
    return result;
  }
  public static FractionalNumber binomialCoefficient(int n, int k)
  {
    return new FractionalNumber(fallingFactorial(n, k), factorial(k));
  }
}



















































