import java.util.ArrayList;
public class Polynomial implements IFunction
{
  // Let n = coefficients.size()-1 be the order of the polynomial.
  // Then this class represents the polynomial
  // x^n*coefficients.get(0) + x^(n-1)*coefficients.get(1)
  // + ... + x*coefficients.get(n-1) + coefficients.get(n)
  private ArrayList<IRealNumber> coefficients;
  public Polynomial(ArrayList<IRealNumber> coefficients)
  {
    this.coefficients = coefficients;
  }
  public IRealNumber compute(IRealNumber input)
  {
    // This uses Horner's method.
    IRealNumber result = coefficients.get(0);
    for (int i = 1; i < coefficients.size(); i++)
    {
      result = new Addition(new Multiply(result, input), coefficients.get(i));
    }
    return result;
  }
  public static IRealNumber approximateDerivative(IRealNumber input, IFunction f, int order, IRealNumber h)
  {
    IRealNumber result = new FractionalConstant(0);
    for (int i = 0; i < order; i+=2)
    {
      result = new Addition(result, new Multiply(f.compute(new Addition(input, new Multiply(new FractionalConstant(order-i), h))), new FractionalConstant(NaturalNumber.binomialCoefficient(order, i).invert())));
    }
    IRealNumber minus = new FractionalConstant(0);
    for (int i = 1; i < order; i+=2)
    {
      result = new Addition(result, new Multiply(f.compute(new Addition(input, new Multiply(new FractionalConstant(order-i), h))), new FractionalConstant(NaturalNumber.binomialCoefficient(order, i).invert())));
    }
    return new Subtraction(result, minus);
  }
  public static Polynomial approximate(IFunction f, int order, IRealNumber h)
  {
    ArrayList<IRealNumber> coefficients = new ArrayList<IRealNumber>();
    for (int i = order; i > 0; i--)
    {
      IRealNumber approximation = approximateDerivative(new FractionalConstant(0), f, i, h);
      coefficients.add(new Multiply(approximation, new FractionalConstant(new NaturalNumber(1), NaturalNumber.factorial(i))));
    }
    coefficients.add(f.compute(new FractionalConstant(0)));
    return new Polynomial(coefficients);
  }
}


























































